"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _isomorphicFetch = require("isomorphic-fetch");

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

var _antd = require("antd");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

//覆盖原生的fetch方法，解决mock不生效的问题
// eslint-disable-next-line
fetch = _isomorphicFetch2.default;
/**
 * 将对象转为'key=value'字符串数组，用来拼接到url上。
 * 对象值为数组也能处理成key=a&key=b
 * @param {Object} params query参数对象
 */
var getQueryParams = function getQueryParams(params) {
    var queryParams = [];
    if (params) {
        var _loop = function _loop(key) {
            var value = params[key];
            if (Array.isArray(value)) {
                value.forEach(function (v) {
                    queryParams.push(key + "=" + encodeURI(v));
                });
            } else if (value !== undefined && value !== "" && value !== null) queryParams.push(key + "=" + encodeURI(params[key]));
        };

        for (var key in params) {
            _loop(key);
        }
    }
    return queryParams;
};
/**
 * 处理请求返回，如果有正确返回内容，转json。如果返回数据为空(204)，返回空的promise；
 * 如果错误返回，做错误处理
 * @param {Object} response 请求的返回数据
 * @param {String/boolean} errorMessage 错误信息
 */
var responseHandle = function responseHandle(response, errorMessage) {
    if (response.ok) {
        //如果返回内容为空，返回一个空的Promise对象，确保能够正确进入then的回调
        if (response.status === 204) return new Promise(function (resolve, reject) {
            resolve();
        });else {
            if (response.headers.get('Content-Type') && response.headers.get('Content-Type').indexOf('application/json') !== -1) {
                return response.json();
            }
            return response.text();
        }
    } else return remoteErrorHandle(errorMessage, response);
};
/**
 * 请求错误的处理。
 * 如果errorMessage为false或者undefined，不显示错误信息；
 * 如果errorMessage指定为字符串，显示指定的错误信息；
 * 如果errorMessage为true，智能显示错误信息：如果请求返回信息中包含errorMessage，显示errorMessage信息；如果没有，显示defaultMessage指定的默认信息；如果未定义，显示“后台错误！”；
 * 不管错误信息是否显示，都会抛出"请求xxx出错"的异常，可以在catch(error)中捕获，并且能够从error.response中拿到请求返回的数据。
 * 抛出异常之后不会再进入then回调。
 * @param {String/boolean} errorMessage 指定错误信息显示
 * @param {Object} response 请求再错误情况下的返回数据
 * @param {String} defaultMessage 指定默认的错误信息
 */
var remoteErrorHandle = function remoteErrorHandle(errorMessage, response, defaultMessage) {
    return new Promise(function (resolve, reject) {
        reject();
        if (response.status === 401) {
            var loginUrl = response.headers.get('loginurl');
            window.location.href = loginUrl;
        } else {
            if (errorMessage === undefined) errorMessage = true;
            if (errorMessage) {
                var info = defaultMessage || "后台错误！";
                if (typeof errorMessage === "string") info = errorMessage;
                response.json().then(function (jsonRes) {
                    if (jsonRes.errorMessage && errorMessage === true) info = jsonRes.errorMessage;
                    _antd.message.error(info);
                }).catch(function (error) {
                    _antd.message.error(info);
                });
            } else {
                var error = new Error("请求:" + response.url + " 出错！");
                error.response = response;
                throw error;
            }
        }
    });
};
var getHttpHeader = function getHttpHeader(headers, url) {
    var baseHttpHeaders = { 'accept': 'application/json', 'Content-Type': 'application/json', 'X-Requested-With': 'XMLHttpRequest' };
    var httpHeaders = Object.assign(baseHttpHeaders, headers);

    // let hash = window.location.hash;
    // if(hash.indexOf("app")>-1){
    //     if(url.indexOf('proxy/aip') > -1 || url.indexOf('proxy/uop') > -1){
    //         let envCode = hash.split("/")[2];
    //         httpHeaders['env-code'] = envCode
    //     }

    // }
    return httpHeaders;
};

var C2Fetch = function () {
    function C2Fetch() {
        _classCallCheck(this, C2Fetch);
    }

    _createClass(C2Fetch, null, [{
        key: "get",
        value: function get(url, params, errorMessage, headers) {
            var queryParams = getQueryParams(params);
            var queryUrl = queryParams.length > 0 ? "?" + queryParams.join("&") : "";
            var httpHeaders = getHttpHeader(headers, url);
            return fetch(this.urlBase + url + queryUrl, { credentials: "include", headers: httpHeaders }).then(function (response) {
                return responseHandle(response, errorMessage);
            });
        }
    }, {
        key: "post",
        value: function post(url, bodyParams, queryParams, errorMessage, headers) {
            var parray = getQueryParams(queryParams);
            var queryUrl = parray.length > 0 ? "?" + parray.join("&") : "";
            return fetch(this.urlBase + url + queryUrl, {
                method: "POST", credentials: "include",
                headers: getHttpHeader(headers),
                body: typeof bodyParams === 'string' ? bodyParams : JSON.stringify(bodyParams)
            }).then(function (response) {
                return responseHandle(response, errorMessage);
            });
        }
    }, {
        key: "put",
        value: function put(url, bodyParams, queryParams, errorMessage, headers) {
            var parray = getQueryParams(queryParams);
            var queryUrl = parray.length > 0 ? "?" + parray.join("&") : "";
            return fetch(this.urlBase + url + queryUrl, {
                method: "PUT", credentials: "include",
                headers: getHttpHeader(headers),
                body: JSON.stringify(bodyParams)
            }).then(function (response) {
                return responseHandle(response, errorMessage);
            });
        }
    }, {
        key: "delete",
        value: function _delete(url, params, errorMessage, headers) {
            var queryParams = getQueryParams(params);
            var queryUrl = queryParams.length > 0 ? "?" + queryParams.join("&") : "";
            return fetch(this.urlBase + url + queryUrl, {
                method: "DELETE", credentials: "include",
                headers: getHttpHeader(headers)
            }).then(function (response) {
                return responseHandle(response, errorMessage);
            });
        }

        /**
         * 
         * @param {String} url 下载url,必须返回一个文件流
         * @param {String} fileName 下载的文件名称
         */

    }, {
        key: "download",
        value: function download(url, fileName, headers) {
            fetch(url, { headers: getHttpHeader(headers) }).then(function (res) {
                return res.blob().then(function (blob) {
                    var link = document.createElement('a');
                    link.href = window.URL.createObjectURL(blob);
                    // var filename = res.headers.get('Content-Disposition');
                    link.download = fileName;
                    link.click();
                    window.URL.revokeObjectURL(link.href);
                });
            });
        }
    }, {
        key: "postFile",
        value: function postFile(url, file, queryParams, errorMessage, headers) {
            if (!headers) {
                headers = {};
            }
            var parray = getQueryParams(queryParams);
            var queryUrl = parray.length > 0 ? "?" + parray.join("&") : "";
            return fetch(this.urlBase + url + queryUrl, {
                method: "POST", credentials: "include",
                headers: Object.assign(headers, { 'X-Requested-With': 'XMLHttpRequest' }),
                body: file
            }).then(function (response) {
                return responseHandle(response, errorMessage);
            });
        }
    }]);

    return C2Fetch;
}();

C2Fetch.urlBase = "";
exports.default = C2Fetch;